function gt2exe, input, addind=addind, gtfunctions=gtfunctions, $
   noss=noss, debug=debug, noind=noind
;+
;   Name: gt2exe
; 
;   Purpose: convert gt function shorthand to valid calls (for use w/execute)
;
;   Input Parameters:
;      input - string of form "<item> <relation> <value>"
;              <item>     = function name OR structure/fits field
;                           {filta, filtb, expmode, dpe, percentd, etc}
;              <relation> = { = , >, <, >=, <=, <>}
;              <value>    = value, list (ex: 1,2,5) or range (ex: 4-10)
;               
;   Examples:
;      IDL> print,gt2exe('dpe=[12-20]',/addind)
;      IDL> gt_dpe(index(ss)) ge 12 AND gt_dpe(index(ss)) le 20
;
;      IDL> print,gt2exe('filta=[2,5]', /addind)
;      IDL> gt_filta(index(ss)) eq 2 OR gt_filta(index(ss)) eq 5
;      IDL> print,gt2exe('crpix1=[256,512,1024]')
;           gt_tagval(index(ss),/crpix1) eq 256 OR
;           gt_tagval(index(ss),/crpix1) eq 512 OR
;           gt_tagval(index(ss),/crpix1) eq 1024
;
;   History:
;      Circa jan-93 (S.L.Freeland) - for spd files
;      22-mar-1995  (SLF) - add range and list (expanded for obs searches)
;      13-jan-1998  (SLF) - make unmapped functions a call to 'gt_tagval'
;                           made ADDIND the default
;      26-Feb-1998  (SLF) - define some default strings (ssw standards)
;      22-apr-1998  (SLF) - added some stuff
;                           if value includes wild card/?? use wc_where  
;-
; map shorthand name to standard functions 'gt_xxx, gtt_yyy'
; TODO - get these from a file (derive from sswloc file)
gt_functions=str2arr('expmode,dp_mode,filta,filtb,res,dpe,percentd,comp')
gtt_functions=str2arr('trace_expdur,trace_res')
gt_stringf=str2arr('wave_name,wave_len,sci_obj,cmp_id,obs_prog,seqid,object,telescop,telescope')

if keyword_set(gtfunctions) then return,'gt_' + gt_functions

strings=[0,0,0,0,0,0,0,0] 		 ; for each function, set if /string used

debug=keyword_set(debug)

; define boolean/symbolic translations
log_operators=	['>=','<=','<>' ,'=','<', '>']
bool_equiv   =  ' ' + ['ge','le','ne' ,'eq','lt','gt'] + ' '  ;blank padded

tinput=strlowcase(strcompress(input,/remove))				; protect input
tinput=str_replace(tinput,'gt_','')	; remove gt root if it is there
					; already (handle both ways)

ind=['','index(ss)'] & str=['',',/string']
case 1 of
   data_chk(addind,/string): index=addind
   keyword_set(noind):index=''
   else: index='index(ss)'
endcase

for i = 0, n_elements(log_operators)-1 do $
   tinput=str_replace(tinput,log_operators(i), bool_equiv(i))

tinput=strcompress(strtrim(tinput,2))	; elminate extraneous blanks

; brackets are optional for list and range values - eliminate them
tinput=str_replace(tinput,'[','')
tinput=str_replace(tinput,']','')

; break the input (<item><relationship><value>)
tinput=str2arr(tinput,' ')
if n_elements(tinput) ne 3 then begin
   box_message,["Illegal format for input, - need a relational opertor",$
		"IDL> exestr=gt2exe('<item> <relation> <value>')"]
   return,tinput
endif


; check for list syntax (ex: [1,4,6]) 
list=where(strpos(tinput,',') ne -1,lcnt)	; comma delimited
if lcnt gt 0 then tinput= $
   arr2str(tinput(0) + ' ' + tinput(1) + ' ' + str2arr(tinput(2)),' OR ')

; expand ranges ex: [1-5] -> ge low AND le hi
range=where(strpos(tinput,'-') ne -1,rcnt)	; dash delimited
if rcnt gt 0 then begin
   rng=str2arr(tinput(range),'-')   
   tinput= tinput(0) + ' ge ' + rng(0) + ' AND ' + $
           tinput(0) + ' le ' + rng(1)
endif

tinput=str_replace(arr2str(tinput,' '),' (','(')

efunc=(strmids(tinput,0,strpos(tinput,' ')))(0)

case 1 of
   is_member(efunc,gt_functions):  tinput=str_replace(tinput,efunc,'gt_'+efunc+'('+ index)
   is_member(efunc,gtt_functions): tinput=str_replace(tinput,efunc,'gtt_'+efunc+'('+index)
   strpos(efunc,'(') ne -1: 
   else: tinput=str_replace(tinput,efunc,'gt_tagval('+index+',/'+efunc)
endcase   

efunc=(strmids(tinput,0,strpos(tinput,' ')))(0)

if strpos(efunc,'(') ne -1 and strmid(efunc,strlen(efunc),1) ne ')' then $
   tinput=str_replace(tinput,efunc,efunc+')')

if keyword_set(noss) then tinput=str_replace(tinput,'(ss)','')

pieces=str2arr(tinput,' ')
np=n_elements(pieces)

for i=1,np-1 do begin
  if total(is_member(strtrim(bool_equiv,2),pieces(i))) gt 0 then begin
    funct=strextract(pieces(i-1),'/',')')
   if total(is_member(gt_stringf,funct)) gt 0  then begin
      pieces(i+1)="'"+pieces(i+1)+"'"       
      if strpos(pieces(i+1),'*') ne -1 or strpos(pieces(i+1),'?') ne -1 then begin
      pieces(i+1)=str_replace(str_replace(pieces(i+1),'*',''),'?','')
      pieces(0)=str_replace(str_replace(pieces(0),'gt_tagval','strpos'),',/','.')
      pieces(0)=str_replace(pieces(0),'strpos(','strpos(strupcase(')+')'
      pieces(i+1)='strupcase('+pieces(i+1)+')'
      pieces(0)=strmid(pieces(0),0,strlen(pieces(0))-1 )+','+pieces(i+1)+')'
      pieces(i:i+1)=['ne','-1']
      endif
     endif
   endif
endfor  

if debug then stop
tinput=arr2str(pieces,' ')

return,tinput

end      



